/*
 * Web Designer
 * Copyright (C) 1999-2000  LADISLAS Willhem
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author's email : zillprog@freesurf.fr
 * 
 * snailmail : 
 * LADISLAS Willhem
 * 2, rue du Serpolet
 * 95490 VAUREAL
 * FRANCE
 */

#include "common_headers.h"
#include "pixmaps/webdesigner-icon.xpm"
#include "pixmaps/logo_wdesign.xpm"

#include "../config.h"

static void flush_queue(void);
static GtkWidget *splash_screen (void);

int
main (int argc, char *argv[])
{
	GtkWidget *vscrollbar;
	GtkWidget *hbox;
	GtkWidget *Nav_bar;
	GtkWidget *Paned;
	GtkWidget *vbox2;
	GdkPixmap *gdk_pixmap;
	GdkBitmap *mask;
	GtkWidget *handle_box;

	GtkWidget *splash_window;
	int show_splash_screen;

	/*
	 * primary initialization 
	 */
	numfileopened = 0;
	numprjfiles = 0;
	clist_row2 = 1;
	lasttag_nb = -1;
	prjsaved = TRUE;
	prjopnd = FALSE;
	prjnew = FALSE;
	instring = FALSE;
	intag = FALSE;
	tab_activ = TRUE;
	splash_window = NULL;
	global.argc = argc;
	global.argv = argv;
	show_splash_screen = TRUE;

	global.tips_window_displayed = FALSE;

	/*
	 * Setup locale
	 */

#ifdef ENABLE_NLS
	setlocale (LC_ALL, "");
	bindtextdomain (PACKAGE, LOCALE_DIR);
	textdomain (PACKAGE);
#endif

	/*
	 * Parse command line...
	 * Step 1
	 */

	if (argc != 1)
	  {
		  if (!strcmp (argv[1], "--version"))
			{
				printf (_ ("\n  Web Designer %s\n\n"), VERSION );
				exit (-1);
			}

		  if (!strcmp (argv[1], "--nosplash"))
		  	{
				show_splash_screen = FALSE;
			}

		  if (!strcmp (argv[1], "--help"))
			{
				printf (_ ("\n  Web Designer %s\n\n"), VERSION );
				printf (_ ("  Synopsys : wdesign [options] [file]...\n\n"));
				printf (_ ("  Options :\n"));
				printf (_ ("  	--help     shows this screen\n"));
				printf (_ ("  	--version  shows the current version of Web Designer\n"));
				printf (_ ("  	--file     Open the file 'file' on Web Designer's startup\n"));
				printf (_ ("  	--project  Open the project 'project' on Web Designer's startup\n"));
				printf (_ ("  	--nosplash start without the splash screen\n"));
				printf ("\n");

				exit (-1);
			}

		  if (argc == 2 && strcmp (argv[1], "--nosplash"))
			{

				printf (_ ("\n  Error : not enough argument\n"));
				printf (_ ("  Type 'wdesign --help' for more informations\n\n"));

				exit (-1);
			}
	  }

	/*
	 * Load saved preferences
	 */
	init_preferences ();
	load_preferences ();

	/*
	 * ToolKits and Library initialization 
	 */

	gtk_init (&argc, &argv);

	/*
	 * Create a splash screen while we load everything else...
	 */
	if ( preferences.splash_screen && show_splash_screen )
	 {
		splash_window = splash_screen ();
	 }
	sprintf (projectname, "%s", _ ("none"));

	/*
	 * Load a fixed font 
	 */
	fixed_font = gdk_fontset_load ("-misc-fixed-medium-r-normal-*-*-120-*-*-c-*-iso8859-1,misc*,-misc*");

	/*
	 * maybe later... we'll be able to user different fonts without trouble 
	 * fixed_font_bold = gdk_fontset_load ("-misc-fixed-bold-r-normal-*-*-120-*-*-c-*-iso8859-1,misc*,-misc*");
	 */

	/*
	 * Get the system colour map and allocate the colour red 
	 */
	cmap = gdk_colormap_get_system ();

	marron.red = 0xc800;
	marron.green = 0x7900;
	marron.blue = 0x3600;
	if (!gdk_color_alloc (cmap, &red))
	  {
		  g_error (_ ("couldn't allocate colour"));
	  }

	red.red = 0xafff;
	red.green = 0;
	red.blue = 0;
	if (!gdk_color_alloc (cmap, &red))
	  {
		  g_error (_ ("couldn't allocate colour"));
	  }

	green.red = 0;
	green.green = 0xafff;
	green.blue = 0;
	if (!gdk_color_alloc (cmap, &green))
	  {
		  g_error (_ ("couldn't allocate colour"));
	  }

	black.red = 0;
	black.green = 0;
	black.blue = 0;
	if (!gdk_color_alloc (cmap, &black))
	  {
		  g_error (_ ("couldn't allocate colour"));
	  }

	blue.red = 0x6700;
	blue.green = 0x5900;
	blue.blue = 0xff00;
	if (!gdk_color_alloc (cmap, &blue))
	  {
		  g_error (_ ("couldn't allocate colour"));
	  }

	/*
	 * Main window of the software 
	 */

	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_signal_connect (GTK_OBJECT (window), "destroy", GTK_SIGNAL_FUNC (close_application), "WM destroy");
	gtk_window_set_title (GTK_WINDOW (window), "Web Designer");

	main_vbox = gtk_vbox_new (FALSE, 1);
	gtk_container_add (GTK_CONTAINER (window), main_vbox);
	gtk_container_border_width (GTK_CONTAINER (main_vbox), 0);
	gtk_widget_show (main_vbox);

	gtk_widget_set_usize (GTK_WIDGET (main_vbox), 640, 460);

	get_main_menu(window, &menubar);

	/*
	 * Menubar in a handlebox
	 */
	handle_box = gtk_handle_box_new ();
	gtk_handle_box_set_shadow_type ( GTK_HANDLE_BOX ( handle_box ), GTK_SHADOW_NONE );
	gtk_box_pack_start (GTK_BOX (main_vbox), handle_box, FALSE, TRUE, 0);
	gtk_widget_show (handle_box);

	gtk_container_add ( GTK_CONTAINER ( handle_box ), menubar );

	gtk_window_set_policy (GTK_WINDOW (window), FALSE, TRUE, TRUE);
	gtk_widget_realize (window);

	if (!getenv ("HOME"))
	  {
		  merror (_ ("Warning : 'HOME' environment variable is not set !"));
	  }
	else
	  {
		  home_directory = (char *) malloc (strlen (getenv ("HOME")) + 20);
		  sprintf (home_directory, "%s%s", getenv ("HOME"), "/.wdesign.menu");
		  gtk_item_factory_parse_rc (home_directory);
		  free (home_directory);
	  }

	/*
	 * Initialize the menu-toolbar...
	 */
	/*
	 * Uncomment this if you want a detachable main toolbar...
	 *   - Algernon
	 */
	
	handle_box = gtk_handle_box_new ();
	gtk_handle_box_set_shadow_type ( GTK_HANDLE_BOX ( handle_box ), GTK_SHADOW_NONE );
	gtk_box_pack_start (GTK_BOX (main_vbox), handle_box, FALSE, FALSE, 0);
	gtk_widget_show ( handle_box );
	gtk_container_add ( GTK_CONTAINER ( handle_box ), load_static_toolbar () );
	
	/* Comment this out if you uncommented the above...
	 *   - Algernon
	 
	gtk_box_pack_start ( GTK_BOX ( main_vbox ), load_static_toolbar(), FALSE, FALSE, 0 );
*/
	/*
	 * Create a new notebook (for the toolbars)
	 */
	handle_box = gtk_handle_box_new ();
	gtk_handle_box_set_shadow_type ( GTK_HANDLE_BOX ( handle_box ), GTK_SHADOW_NONE );
	gtk_box_pack_start (GTK_BOX (main_vbox), handle_box, FALSE, FALSE, 0);
	gtk_widget_show ( handle_box );
	
	notebook0 = gtk_notebook_new ();

	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook0), GTK_POS_TOP);
	gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook0), 1);
	gtk_notebook_popup_enable (GTK_NOTEBOOK (notebook0));
	gtk_notebook_set_tab_vborder (GTK_NOTEBOOK (notebook0), 0);

	gtk_container_add ( GTK_CONTAINER ( handle_box ), notebook0 );
	gtk_widget_show (notebook0);

	/*
	 * initialize the popup menu
	 */
	init_popmenu ();

	/*
	 * Initialize the toolbar
	 */
	load_first_toolbar ();

	/*
	 * Vertical Paned window 
	 */
	Paned = gtk_vpaned_new ();

	/*
	 * gtk_paned_set_handle_size(GTK_PANED(Paned),10);
	 * gtk_paned_set_gutter_size(GTK_PANED(Paned),15); 
	 */
	gtk_widget_show (Paned);

	vbox2 = gtk_vbox_new (FALSE, 1);
	gtk_paned_pack1 (GTK_PANED (Paned), vbox2, FALSE, FALSE);
	gtk_widget_show (vbox2);

	/*
	 * Create a new notebook 
	 */
	notebook = gtk_notebook_new ();
	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
	gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), 1);
	gtk_notebook_popup_enable (GTK_NOTEBOOK (notebook));
	gtk_notebook_set_tab_vborder (GTK_NOTEBOOK (notebook), 0);
	gtk_box_pack_start (GTK_BOX (vbox2), notebook, TRUE, TRUE, 0);
	gtk_widget_show (notebook);

	/*
	 * Navigation bar 
	 */

	Nav_bar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
	gtk_box_pack_start (GTK_BOX (vbox2), Nav_bar, FALSE, TRUE, 0);
	gtk_toolbar_set_button_relief (GTK_TOOLBAR (Nav_bar), GTK_RELIEF_NONE);

	gtk_toolbar_set_space_size (GTK_TOOLBAR (Nav_bar), 10);
	gtk_toolbar_append_space (GTK_TOOLBAR (Nav_bar));

	gtk_toolbar_set_space_style (GTK_TOOLBAR (Nav_bar), GTK_TOOLBAR_SPACE_LINE);

	gtk_toolbar_append_item (GTK_TOOLBAR (Nav_bar), NULL, _ ("Move to HEAD tag"),
					   "", new_pixmap (126, window->window, &color),
					 GTK_SIGNAL_FUNC (move_in_document), (int *) DOC_MOVE_HEAD);

	gtk_toolbar_append_item (GTK_TOOLBAR (Nav_bar), NULL, _ ("Move to BODY tag"),
					   "", new_pixmap (127, window->window, &color),
					 GTK_SIGNAL_FUNC (move_in_document), (int *) DOC_MOVE_BODY);

	gtk_toolbar_append_item (GTK_TOOLBAR (Nav_bar), NULL, _ ("Move to bottom of document"),
					   "", new_pixmap (128, window->window, &color),
					 GTK_SIGNAL_FUNC (move_in_document), (int *) DOC_MOVE_BOTTOM);

	gtk_toolbar_append_item (GTK_TOOLBAR (Nav_bar), NULL, _ ("Move to top of document"),
					   "", new_pixmap (129, window->window, &color),
					 GTK_SIGNAL_FUNC (move_in_document), (int *) DOC_MOVE_TOP);

	gtk_toolbar_append_space (GTK_TOOLBAR (Nav_bar));

	gtk_toolbar_append_item (GTK_TOOLBAR (Nav_bar), NULL, _ ("Goto first document"),
					   "", new_pixmap (110, window->window, &color),
				   GTK_SIGNAL_FUNC (move_notebook_page), (int *) DOC_GOTO_FIRST);

	gtk_toolbar_append_item (GTK_TOOLBAR (Nav_bar), NULL, _ ("Previous document"),
					   "", new_pixmap (111, window->window, &color),
				   GTK_SIGNAL_FUNC (move_notebook_page), (int *) DOC_GOTO_PREV);

	gtk_toolbar_append_item (GTK_TOOLBAR (Nav_bar), NULL, _ ("Next document"),
					   "", new_pixmap (112, window->window, &color),
				   GTK_SIGNAL_FUNC (move_notebook_page), (int *) DOC_GOTO_NEXT);

	gtk_toolbar_append_item (GTK_TOOLBAR (Nav_bar), NULL, _ ("Goto last document"),
					   "", new_pixmap (113, window->window, &color),
				   GTK_SIGNAL_FUNC (move_notebook_page), (int *) DOC_GOTO_LAST);

	gtk_toolbar_append_space (GTK_TOOLBAR (Nav_bar));

	gtk_widget_show (Nav_bar);

	/*
	 * create the box for the tooltip window 
	 */
	hbox2 = gtk_hbox_new (FALSE, 1);

	/*
	 * Create the tag info window 
	 */
	tag_window = gtk_text_new (NULL, NULL);
	gtk_text_freeze (GTK_TEXT (tag_window));

	vscrollbar = gtk_vscrollbar_new (GTK_TEXT (tag_window)->vadj);

	gtk_text_thaw (GTK_TEXT (tag_window));
	gtk_box_pack_start (GTK_BOX (hbox2), tag_window, TRUE, TRUE, 2);
	gtk_box_pack_start (GTK_BOX (hbox2), vscrollbar, FALSE, FALSE, 2);
	widget_style (GTK_WIDGET (tag_window));

	gtk_widget_set_usize (GTK_WIDGET (hbox2), 50, 85);
	gtk_paned_pack2 (GTK_PANED (Paned), hbox2, FALSE, FALSE);
	gtk_widget_show_all (hbox2);

	gtk_box_pack_start (GTK_BOX (main_vbox), Paned, TRUE, TRUE, 2);

	/*
	 * status bars 
	 */
	hbox = gtk_hbox_new (FALSE, 1);

	/*
	 * Create a new status bar 
	 */
	status_bar = gtk_statusbar_new ();
	gtk_box_pack_start (GTK_BOX (hbox), status_bar, TRUE, TRUE, 10);
	gtk_widget_show (status_bar);
	context_id = gtk_statusbar_get_context_id (GTK_STATUSBAR (status_bar), "Status_bar");

	/*
	 * Create a new status bar 
	 */
	file_status_bar = gtk_statusbar_new ();
	gtk_box_pack_end (GTK_BOX (hbox), file_status_bar, FALSE, TRUE, 10);
	gtk_widget_show (file_status_bar);
	context_id = gtk_statusbar_get_context_id (GTK_STATUSBAR (file_status_bar), "file_status_bar");

	/*
	 * Create a new status bar 
	 */
	coord_status_bar = gtk_statusbar_new ();
	gtk_box_pack_end (GTK_BOX (hbox), coord_status_bar, FALSE, FALSE, 2);
	gtk_widget_show (coord_status_bar);
	context_id = gtk_statusbar_get_context_id (GTK_STATUSBAR (coord_status_bar), "coord_status_bar");

	gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, TRUE, 2);
	gtk_widget_show (hbox);

	/*
	 * gtk_paned_pack2(GTK_PANED(Paned), vbox2, FALSE, TRUE);
	 * gtk_widget_show(vbox2);
	 * 
	 * gtk_paned_set_position(GTK_PANED(Paned),170);
	 */
	 
	gtk_statusbar_push (GTK_STATUSBAR (coord_status_bar), context_id, _ (" Line : 0, Column : 0 "));

	gtk_signal_connect_after (GTK_OBJECT (notebook), "switch_page", GTK_SIGNAL_FUNC (gtk_swpg), NULL);

	new_html_file ();

	gdk_pixmap = gdk_pixmap_create_from_xpm_d ( window->window, &mask, NULL, webdesigner_icon_xpm );
	gdk_window_set_icon ( window->window, NULL, gdk_pixmap, mask );

	apply_preferences ();

	/*
	 * Parse command line...
	 * Step 2
	 */

	if (argc != 1)
	  {
		  if (!strcmp (argv[1], "--file"))
			{
				int i;
				close_file ();
				for (i = 2; i < argc; i++)
				  {
				  	if ( openfile2 (argv[i], FALSE, FALSE) == 0 )
					  {
					  	new_html_file ();
					  };
				  }
			}

		  if (!strcmp (argv[1], "--project"))
			{
				close_file ();
				if (open_project2 (argv[2]) == -1)
					new_html_file ();
			}
	  }

	/*
	 * Kill splash screen...
	 */

	if ( preferences.splash_screen && show_splash_screen )
	 {
		flush_queue();
		sleep ( 2 );
		gtk_signal_handlers_destroy (GTK_OBJECT(splash_window));
		gtk_widget_destroy (GTK_WIDGET(splash_window));
	 }

	/*
	 * Display main window
	 */

	gtk_widget_show_all (window);

	/*
	 * Display tips ...
	 */
	if ( preferences.show_tips )
	  {
		  display_tip_window();
	  }

	gtk_widget_grab_focus (sheet[curfile].text);

	gtk_main ();

	return (0);
}

/*
 * The following two functions are borrowsed from bluefish...
 *      - Algernon
 */

void flush_queue(void)
{
	while ( gtk_events_pending () )
	{
		gtk_main_iteration();
	}
	while ( gdk_events_pending () )
	{
		gdk_flush();
	}
}

GtkWidget *splash_screen (void)
{
	GtkWidget *vbox, *splash_window;
	GtkWidget *wpixmap;
	GdkPixmap *pixm;
	char title[64];

	sprintf ( title, "Web Designer %s", VERSION );
	splash_window = gtk_window_new (GTK_WINDOW_POPUP);
	gtk_window_set_title ( GTK_WINDOW (splash_window), title );
	gtk_window_position ( GTK_WINDOW (splash_window), GTK_WIN_POS_CENTER);
	vbox=gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (splash_window), vbox);
	gtk_widget_show ( vbox );
	gtk_widget_realize ( splash_window );
	pixm = gdk_pixmap_create_from_xpm_d ( splash_window->window, NULL, NULL, logo_wdesign_xpm);
	wpixmap = gtk_pixmap_new (pixm, NULL);
	gdk_pixmap_unref(pixm);
	gtk_box_pack_start (GTK_BOX (vbox), wpixmap, FALSE, FALSE, 0);
	gtk_widget_show (wpixmap);
	gtk_widget_show (splash_window);
	flush_queue();
	return splash_window;
}
